// Prot.cpp : Defines the entry point for the console application.
//

#define WIN32_LEAN_AND_MEAN
#define _CRT_SECURE_NO_DEPRECATE
#define STRICT
#define MAINPROG

#include <windows.h>
#include <tchar.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif
#include "disasm/disasm.h"
#ifdef __cplusplus
}
#endif

#pragma warning(disable:4311 4267)
#pragma comment(linker, "/SECTION:.text,RWE")

#define BASE_LOW  0x10010000
#define BASE_HIGH 0x40000000

#define RAND_LOW 0xA0000000
#define RAND_HIGH 0xFECABDFF

#define MakePtr( cast, ptr, addValue ) (cast)( (ULONG_PTR)(ptr) + (DWORD)(addValue))

#define PAGE_SIZE 4096

char *regs[] = {"eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi"};

#define PUSH_REG	5
#define POP_REG		6
#define PUSH_IMM	7
#define MOV_REG		8
#define SUB_REG		1
#define ADD_REG		0
#define XOR_REG		2
#define ROL_REG		3
#define ROR_REG		4
#define JMP_REG		9
#define RET			10

char *opsStr[] = {"add %s,0%x", "sub %s,0%x", "xor %s,0%x", "rol %s,0%x", "ror %s,0%x", "push %s", "pop %s", "push 0%x", "mov %s,0%x", "jmp %s", "ret"};

//these are reversed, since if the operation is an add, to decrypt we need a sub, so OP_ADD indexes "sub %s,0%x"
#define OP_ADD 1
#define OP_SUB 0
#define OP_XOR 2
#define OP_ROL 4
#define OP_ROR 3
#define SWITCH_REG 5

#define REG_EAX 0
#define REG_ECX 1
#define REG_EDX 2
#define REG_EBX 3
#define REG_ESP 4
#define REG_EBP 5
#define REG_ESI 6
#define REG_EDI 7

typedef struct _OpInfo
{
	ULONG Operation;
	ULONG Operand;
} OpInfo;

typedef struct _PEInfo
{
	HANDLE hPEFile;
	HANDLE hMappedFile;
	LPVOID FileBase;
	PIMAGE_DOS_HEADER ImageDosHeader;
	PIMAGE_NT_HEADERS ImageNtHeaders;
	PIMAGE_FILE_HEADER ImageFileHeader;
	PIMAGE_OPTIONAL_HEADER ImageOptionalHeader;

	PIMAGE_SECTION_HEADER WorkingSection;
} PEInfo;

typedef struct _LoaderInfo
{
	PEInfo *pPEInfo;
	DWORD OriginalRelocRVA;
	DWORD CurrentOffset;
	DWORD OriginalITRVA;
	DWORD OriginalETRVA;
	DWORD TextRVA;
	DWORD TextSize;
	DWORD DataRVA;
	DWORD DataSize;
	DWORD RDataRVA;
	DWORD RDataSize;
} LoaderInfo;

void UnloadPE(PEInfo *pPEInfo);

#ifdef __cplusplus
extern "C" {
#endif
DWORD __stdcall RvaToOffset(DWORD Rva, PIMAGE_NT_HEADERS ImageNtHeaders);
DWORD __stdcall OffsetToRva(DWORD Offset, PIMAGE_NT_HEADERS ImageNtHeaders);
PIMAGE_SECTION_HEADER __stdcall GetEnclosingSectionHeader(DWORD rva, PIMAGE_NT_HEADERS pNTHeader);
DWORD __stdcall RoundUp(DWORD value, DWORD base);
DWORD __stdcall IntervalRand(DWORD Low, DWORD High);
BOOL __stdcall ContinueWork(LoaderInfo *pInfo);
DWORD __stdcall Build1Loader(PEInfo *pPEInfo, DWORD LdrOffset, DWORD JmpAddress);
DWORD __stdcall Build15Loader(PEInfo *pPEInfo, DWORD LdrOffset, DWORD RealLdrAddress, DWORD RealLdrSize, DWORD RealLdrDSize);
BOOL __stdcall EncryptCode(PEInfo *pPEInfo, PUCHAR Key);
BOOL __stdcall EncryptData(PEInfo *pPEInfo, PUCHAR Key);
BOOL __stdcall EncryptRData(PEInfo *pPEInfo, PUCHAR Key);
#ifdef __cplusplus
}
#endif

DWORD AssemblePush(PUCHAR Offset, DWORD Value, BOOL Reg);
DWORD AssemblePop(PUCHAR Offset, DWORD Register);
DWORD AssembleRet(PUCHAR Offset, DWORD Register);

void encipher(DWORD *v, DWORD *w, DWORD *k);
void decipher(DWORD *v, DWORD *w, DWORD *k);

LoaderInfo g_LoaderInfo;
DWORD g_dwOriginalRawSize;
DWORD g_dwNewRawSize;

/***
 *	Select a random register, excluding ESP
 ***/
BYTE SelectRandRegister()
{
	BYTE _register = 0x00;

	do
	{
		_register = rand() % 8;	// Select register from EAX to EDI
	} while (_register == REG_ESP);

	return _register;
}

/**
 * _garbage_jmp
 * Alter the register used in _short_jmp MACRO with a random register
 * EA introduce the [JMP] FAR opcode, the operand of JMP is the CPU instruction	
 * @return Boolean value on successful
 */
bool _garbage_jmp(UCHAR* ptr)
{
	// Alter 0x00, 0x09, 0x0A, 0x0C, 0x0E
	static const unsigned char pattern[] = 
	{
		0x50,							// 0x00	PUSH
		0xE8, 0x03, 0x00, 0x00, 0x00,	// 0x01	CALL [0x09]
		0xEA,							// 0x06	[JMP FAR] -> SKIP
		0xEB, 0x05,						// 0x07	JMP SHORT
		0x58,							// 0x09 POP
		0x40,							// 0x0A INC
		0xFF, 0xE0,						// 0x0B JMP REG [0X07]
		0xEA,							// 0x0D [JMP FAR] -> SKIP
		0x58							// 0x0E POP
	};

	if (memcmp(ptr, pattern, sizeof(pattern)) == 0) {
		BYTE _register = SelectRandRegister();

		ptr[0x00] |= _register;	// PUSH register
		ptr[0x09] |= _register;
		ptr[0x0A] |= _register;
		ptr[0x0C] |= _register;
		ptr[0x0E] |= _register;
		
		return true;
	}

	return false;
}

PIMAGE_SECTION_HEADER __stdcall GetEnclosingSectionHeader(DWORD rva, PIMAGE_NT_HEADERS pNTHeader)
{
	PIMAGE_SECTION_HEADER section = IMAGE_FIRST_SECTION(pNTHeader);
    unsigned i;
    
    for ( i=0; i < pNTHeader->FileHeader.NumberOfSections; i++)
    {
        if ( (rva >= section[i].VirtualAddress) && 
             (rva < (section[i].VirtualAddress + section[i].Misc.VirtualSize)))
            return &section[i];
    }
    
    return 0;
}

//
// Calcola la posizione Rva nel corrispettivo file
//
DWORD __stdcall RvaToOffset(DWORD Rva, PIMAGE_NT_HEADERS ImageNtHeaders)
{
	DWORD Offset = Rva;
	IMAGE_SECTION_HEADER *Img;

	Img = IMAGE_FIRST_SECTION(ImageNtHeaders);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (int i = 0; i < ImageNtHeaders->FileHeader.NumberOfSections; i++)
	{
		if (Rva >= Img[i].VirtualAddress &&
			Rva < (Img[i].VirtualAddress +
			Img[i].SizeOfRawData))
		{
			Offset -= Img[i].VirtualAddress;
			Offset += Img[i].PointerToRawData;

			return Offset;
		}
	}

	return NULL;

}

DWORD __stdcall OffsetToRva(DWORD Offset, PIMAGE_NT_HEADERS ImageNtHeaders)
{
	DWORD Rva = Offset, Limit, Start;
	IMAGE_SECTION_HEADER *Img;

	Img = IMAGE_FIRST_SECTION(ImageNtHeaders);

	if (Offset < Img->PointerToRawData)
		return Offset;

	for (int i = 0; i < ImageNtHeaders->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].PointerToRawData)
			Start = Img[i].PointerToRawData;
		else
			Start = Img[i].VirtualAddress;

		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Offset >= Start && Offset < (Start + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].PointerToRawData;
				Offset += Img[i].VirtualAddress;
			}

			return Offset;
		}
	}
	return NULL;
}

DWORD __stdcall RoundUp(DWORD value, DWORD base)
{
	DWORD tmpValue = value;
	while((tmpValue % base) != 0)
		tmpValue++;
	return tmpValue;
}

DWORD __stdcall IntervalRand(DWORD Low, DWORD High)
{
	DWORD dwRand;

	dwRand = (unsigned int)(Low + (double)rand() * (High - Low + 1) / RAND_MAX);

	return dwRand;
}

PEInfo *LoadPE(TCHAR *FileName)
{
	PEInfo *pPEInfo;

	if(FileName == NULL)
	{
		_tprintf(_T("LoadPE: No input file specified.\n"));
		return NULL;
	}

	pPEInfo = (PEInfo *)VirtualAlloc(NULL, sizeof(PEInfo), MEM_COMMIT, PAGE_READWRITE);
	if(pPEInfo == NULL)
	{
		_tprintf(_T("LoadPE: Could not allocate %d bytes of memory.\n"), sizeof(PEInfo));
		return NULL;
	}

	pPEInfo->hPEFile = CreateFile(FileName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
	if(pPEInfo->hPEFile == INVALID_HANDLE_VALUE)
	{
		_tprintf(_T("LoadPE: Could not open %s.\n"), FileName);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->hMappedFile = CreateFileMapping(pPEInfo->hPEFile, NULL, PAGE_READWRITE, NULL, GetFileSize(pPEInfo->hPEFile, NULL) + 4*PAGE_SIZE, NULL);
	if(pPEInfo->hMappedFile == INVALID_HANDLE_VALUE)
	{
		_tprintf(_T("LoadPE: Could not create file mapping, handle = %08x.\n"), (ULONG_PTR)pPEInfo->hPEFile);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->FileBase = MapViewOfFile(pPEInfo->hMappedFile, FILE_MAP_READ | FILE_MAP_WRITE, NULL, NULL, 0);
	if(pPEInfo->FileBase == NULL)
	{
		_tprintf(_T("LoadPE: Could not map file into memory, map handle = %08x.\n"), (ULONG_PTR)pPEInfo->hMappedFile);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->ImageDosHeader = (PIMAGE_DOS_HEADER)pPEInfo->FileBase;
	if(pPEInfo->ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		_tprintf(_T("LoadPE: %s is not a valid pe file.\n"), FileName);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->ImageNtHeaders = MakePtr(PIMAGE_NT_HEADERS, pPEInfo->FileBase, pPEInfo->ImageDosHeader->e_lfanew);
	if(pPEInfo->ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		_tprintf(_T("LoadPE: %s is not a valid pe file.\n"), FileName);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->ImageFileHeader = &pPEInfo->ImageNtHeaders->FileHeader;
	pPEInfo->ImageOptionalHeader = &pPEInfo->ImageNtHeaders->OptionalHeader;

	PIMAGE_SECTION_HEADER pSection = IMAGE_FIRST_SECTION(pPEInfo->ImageNtHeaders);
	for(ULONG i = 0; i < pPEInfo->ImageFileHeader->NumberOfSections; i++)
	{
		if(strncmp((char *)pSection[i].Name, ".text", 5) == 0)
		{
			g_LoaderInfo.TextRVA = pSection[i].VirtualAddress;
			g_LoaderInfo.TextSize = pSection[i].SizeOfRawData;
		}
		else if(strncmp((char *)pSection[i].Name, ".data", 5) == 0)
		{
			g_LoaderInfo.DataRVA = pSection[i].VirtualAddress;
			g_LoaderInfo.DataSize = pSection[i].SizeOfRawData;
		}
		else if(strncmp((char *)pSection[i].Name, ".rdata", 6) == 0)
		{
			g_LoaderInfo.RDataRVA = pSection[i].VirtualAddress;
			g_LoaderInfo.RDataSize = pSection[i].SizeOfRawData;
		}
		else if(strncmp((char *)pSection[i].Name, ".reloc", 6) == 0)
			pPEInfo->WorkingSection = &pSection[i];

		pSection[i].Characteristics |= IMAGE_SCN_MEM_WRITE; //all sections writeable
	}

	g_LoaderInfo.CurrentOffset = (DWORD)((PUCHAR)pPEInfo->WorkingSection->PointerToRawData + pPEInfo->WorkingSection->SizeOfRawData);

	pPEInfo->WorkingSection->SizeOfRawData += 4*PAGE_SIZE;
	pPEInfo->WorkingSection->SizeOfRawData = RoundUp(pPEInfo->WorkingSection->SizeOfRawData, pPEInfo->ImageOptionalHeader->FileAlignment);
	pPEInfo->WorkingSection->Misc.VirtualSize += 4*PAGE_SIZE;

	pPEInfo->WorkingSection->Characteristics |= IMAGE_SCN_MEM_EXECUTE | IMAGE_SCN_CNT_CODE;
	pPEInfo->WorkingSection->Characteristics -= IMAGE_SCN_MEM_DISCARDABLE;

	pPEInfo->ImageOptionalHeader->SizeOfImage += 4*PAGE_SIZE;

	g_LoaderInfo.OriginalITRVA = pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress;
	g_LoaderInfo.OriginalETRVA = pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress;

	//if load config directory is not 0, SEH inside the loader will not work, is related to RtlLookupFunctionTable inside ntdll.dll
	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG].VirtualAddress = 0;
	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG].Size = 0;
	return pPEInfo;
}

void UnloadPE(PEInfo *pPEInfo)
{
	if(pPEInfo != NULL)
	{
		if(pPEInfo->FileBase != NULL)
			UnmapViewOfFile(pPEInfo->FileBase);

		if(pPEInfo->hMappedFile != INVALID_HANDLE_VALUE)
			CloseHandle(pPEInfo->hMappedFile);

		if(pPEInfo->hPEFile != INVALID_HANDLE_VALUE)
			CloseHandle(pPEInfo->hPEFile);

		VirtualFree(pPEInfo, sizeof(PEInfo), MEM_RELEASE);
	}
}

// Ricalcola gli offsett per la nuova ImageBase
// partendo dalla .reloc
//
BOOL RelocateFile(PEInfo *pPEInfo)
{
	DWORD dwNewImageBase;
	DWORD dwDelta;
	DWORD dwRelocOffset;
	DWORD dwNumReloc;
	WORD *wData;
	PIMAGE_BASE_RELOCATION ImageBaseReloc;

	if(pPEInfo == NULL)
		return FALSE;

	dwNewImageBase = IntervalRand(BASE_LOW, BASE_HIGH);
	dwNewImageBase = RoundUp(dwNewImageBase, 0x00100000);

	dwDelta = dwNewImageBase - pPEInfo->ImageOptionalHeader->ImageBase;
	dwRelocOffset = RvaToOffset(pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress, pPEInfo->ImageNtHeaders);

	ImageBaseReloc = MakePtr(PIMAGE_BASE_RELOCATION, pPEInfo->FileBase, dwRelocOffset);

	do
	{
		if(ImageBaseReloc->SizeOfBlock == 0)
			break;

		dwNumReloc = (ImageBaseReloc->SizeOfBlock - IMAGE_SIZEOF_BASE_RELOCATION) / sizeof(WORD);
		wData = MakePtr(WORD *, ImageBaseReloc, IMAGE_SIZEOF_BASE_RELOCATION);

		for(ULONG i = 0; i < dwNumReloc; i++)
		{
			DWORD offset = (*wData & 0xFFF) + ImageBaseReloc->VirtualAddress;

			BYTE type = (BYTE)(*wData >> 12);
			if(type != IMAGE_REL_BASED_ABSOLUTE)
			{
				DWORD blockOffset = RvaToOffset(offset, pPEInfo->ImageNtHeaders);
				DWORD *block = MakePtr(DWORD *, pPEInfo->FileBase, blockOffset);

				*block += dwDelta;
			}

			wData++;
		}

		ImageBaseReloc = (PIMAGE_BASE_RELOCATION)wData;
	} while(*(DWORD *)wData != 0);

	pPEInfo->ImageOptionalHeader->ImageBase = dwNewImageBase;

	return TRUE;
}

BOOL EncryptReloc(PEInfo *pPEInfo)
{
	DWORD *ptrLeft;

	if(pPEInfo == NULL)
		return FALSE;

	// Per Bombare IDA e soci...
	pPEInfo->ImageOptionalHeader->LoaderFlags = IntervalRand(RAND_LOW, RAND_HIGH);
	pPEInfo->ImageOptionalHeader->NumberOfRvaAndSizes = IntervalRand(RAND_LOW, RAND_HIGH);

	g_LoaderInfo.OriginalRelocRVA = pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress;

	// Punta alla .reloc
	ptrLeft = MakePtr(DWORD *, pPEInfo->FileBase, RvaToOffset(pPEInfo->WorkingSection->VirtualAddress, pPEInfo->ImageNtHeaders));

	while(*ptrLeft != 0)
	{
		*ptrLeft ^= pPEInfo->ImageOptionalHeader->LoaderFlags;
		*ptrLeft ^= pPEInfo->ImageOptionalHeader->NumberOfRvaAndSizes;
		ptrLeft++;
	}

	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress = 0;
	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size = 0;
	return TRUE;
}

BOOL ProcessExportTable(PEInfo *pPEInfo)
{
	PIMAGE_EXPORT_DIRECTORY newExportTable;
	PIMAGE_EXPORT_DIRECTORY origExportTable;
	DWORD etOffset;
	DWORD newEtOffset;
	DWORD offset;
	DWORD rva;
	PCHAR oldPtr;
	PCHAR curPtr;

	if(pPEInfo == NULL)
		return FALSE;

	etOffset = RvaToOffset(pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress, pPEInfo->ImageNtHeaders);
	origExportTable = MakePtr(PIMAGE_EXPORT_DIRECTORY, pPEInfo->FileBase, etOffset);

	newEtOffset = IntervalRand(g_LoaderInfo.CurrentOffset, g_LoaderInfo.CurrentOffset + 100);
	newExportTable = MakePtr(PIMAGE_EXPORT_DIRECTORY, pPEInfo->FileBase, newEtOffset);

	memcpy(newExportTable, origExportTable, sizeof(IMAGE_EXPORT_DIRECTORY));
	curPtr = MakePtr(PCHAR, newExportTable, sizeof(IMAGE_EXPORT_DIRECTORY));

	offset = RvaToOffset(origExportTable->AddressOfFunctions, pPEInfo->ImageNtHeaders);
	oldPtr = MakePtr(PCHAR, pPEInfo->FileBase, offset);

	memcpy(curPtr, oldPtr, sizeof(DWORD)*origExportTable->NumberOfFunctions);
	rva = OffsetToRva((DWORD)((PCHAR)curPtr - (PCHAR)pPEInfo->FileBase), pPEInfo->ImageNtHeaders);

	newExportTable->AddressOfFunctions = rva;

	curPtr = MakePtr(PCHAR, curPtr, sizeof(DWORD) * origExportTable->NumberOfFunctions);
	offset = RvaToOffset(origExportTable->AddressOfNames, pPEInfo->ImageNtHeaders);
	oldPtr = MakePtr(PCHAR, pPEInfo->FileBase, offset);

	memcpy(curPtr, oldPtr, sizeof(DWORD) * origExportTable->NumberOfFunctions);

	rva = OffsetToRva((DWORD)((PCHAR)curPtr - (PCHAR)pPEInfo->FileBase), pPEInfo->ImageNtHeaders);
	newExportTable->AddressOfNames = rva;

	curPtr = MakePtr(PCHAR, curPtr, sizeof(DWORD) * origExportTable->NumberOfFunctions);
	offset = RvaToOffset(origExportTable->AddressOfNameOrdinals, pPEInfo->ImageNtHeaders);
	oldPtr = MakePtr(PCHAR, pPEInfo->FileBase, offset);

	memcpy(curPtr, oldPtr, sizeof(WORD) * origExportTable->NumberOfNames);

	rva = OffsetToRva((DWORD)((PCHAR)curPtr - (PCHAR)pPEInfo->FileBase), pPEInfo->ImageNtHeaders);
	newExportTable->AddressOfNameOrdinals = rva;

	rva = OffsetToRva((DWORD)((PCHAR)newExportTable - (PCHAR)pPEInfo->FileBase), pPEInfo->ImageNtHeaders);

	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress = rva;

	curPtr += newExportTable->NumberOfNames * sizeof(WORD);
	*curPtr = 0;
	curPtr++;

	g_LoaderInfo.CurrentOffset = (DWORD)(curPtr - (PCHAR)pPEInfo->FileBase);

	rva = OffsetToRva((DWORD)((PCHAR)newExportTable - (PCHAR)pPEInfo->FileBase), pPEInfo->ImageNtHeaders);
	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress = rva;
	return TRUE;
}

/**
 * AssemblePush
 * randomly generates a "push reg" or "push value" variant and assemble it 
 */
DWORD AssemblePush(PUCHAR Offset, DWORD Value, BOOL Reg)
{
	DWORD rnd;
	t_asmmodel am;
	PUCHAR offPtr = Offset;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	rnd = IntervalRand(0, 100);
	if(rnd < 50)
	{
		if(Reg) {
			offPtr[0] = 0xEB;	// jmp short
			offPtr[1] = 0xFF;	// -1
			offPtr[2] = 0xF0 | Value;	// OR register
			offPtr += 3;
			// preserve original assemble instruction
			//sprintf(instr, opsStr[PUSH_REG], regs[Value]);
		} else {
			sprintf(instr, opsStr[PUSH_IMM], Value);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;
		}
	}
	else
	{
		sprintf(instr, opsStr[SUB_REG], regs[4], 4);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;

		if(Reg)
			sprintf(instr, "mov [esp], %s", regs[Value]);
		else
			sprintf(instr, "mov [dword esp], 0%x", Value);

		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	return (DWORD)(offPtr - Offset);
}

DWORD AssemblePop(PUCHAR Offset, DWORD Register)
{
	DWORD rnd;
	t_asmmodel am;
	PUCHAR offPtr = Offset;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	rnd = IntervalRand(0, 100);
	if(rnd < 50)
	{
		sprintf(instr, opsStr[POP_REG], regs[Register]);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}
	else
	{
		sprintf(instr, "mov %s, [esp]", regs[Register]);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;

		sprintf(instr, opsStr[ADD_REG], regs[4], 4);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	return (DWORD)(offPtr - Offset);
}

//this randomly generates a ret variant and assemble it
DWORD AssembleRet(PUCHAR Offset, DWORD Register)
{
	t_asmmodel am;
	PUCHAR offPtr = Offset;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	//rnd = IntervalRand(0, 100);
	//if(rnd < 50) //generate a jmp reg
	//{
		sprintf(instr, opsStr[JMP_REG], regs[Register]);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	//}
	//else
	//{
	//	offPtr += AssemblePush(offPtr, Register, TRUE);

	//	sprintf(instr, "RETN");
	//	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	//	//memcpy(offPtr, am.code, am.length);
	//	*offPtr = 0xC3;
	//	offPtr += 1;
	//	//offPtr += am.length;
	//}

	return (DWORD)(offPtr - Offset);
}

DWORD __stdcall Build1Loader(PEInfo *pPEInfo, DWORD LdrOffset, DWORD JmpAddress)
{
	OpInfo *ops;
	t_asmmodel am;
	ULONG numOps;
	DWORD curAddress = JmpAddress;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	numOps = IntervalRand(1, 10); //maximum number of operations
	ops = (OpInfo *)malloc(sizeof(OpInfo) * numOps);

	//manipulate JmpAddress with reversible operations
	for(ULONG i = 0; i < numOps; i++)
	{
		ULONG op = IntervalRand(0, 5);
		ops[i].Operation = op;

		switch(op)
		{
			case OP_ADD:
			{
				DWORD rnd = (DWORD)rand();
				ops[i].Operand = rnd;
				__asm {
					push eax
					mov eax, curAddress
					add eax, rnd
					mov curAddress, eax
					pop eax
				}
			}
			break;

			case OP_SUB:
			{
				DWORD rnd = (DWORD)rand();
				ops[i].Operand = rnd;
				__asm {
					push eax
					mov eax, curAddress
					sub eax, rnd
					mov curAddress, eax
					pop eax
				}
			}
			break;

			case OP_XOR:
			{
				DWORD rnd = (DWORD)rand();
				ops[i].Operand = rnd;
				curAddress ^= rnd;
			}
			break;

			case OP_ROL:
			{
				BYTE rnd = (BYTE)IntervalRand(1, 16);
				ops[i].Operand = (DWORD)rnd;
				__asm {
					push eax
					push ecx
					mov eax, curAddress
					movzx ecx, rnd
					rol eax, cl
					mov curAddress, eax
					pop ecx
					pop eax
				}
			}
			break;

			case OP_ROR:
			{
				BYTE rnd = (BYTE)IntervalRand(1, 16);
				ops[i].Operand = (DWORD)rnd;
				__asm {
					push eax
					push ecx
					mov eax, curAddress
					movzx ecx, rnd
					ror eax, cl
					mov curAddress, eax
					pop ecx
					pop eax
				}
			}
			break;
		}
	}

	DWORD workRegister = 0;

	do
	{
		workRegister = IntervalRand(0, 7);
	} while(workRegister == REG_ESP); //esp can't be used

	PUCHAR offPtr = (PUCHAR)LdrOffset;

#ifdef _DEBUG
	*offPtr = 0xCC;
	offPtr += 1;
#endif

	offPtr += AssemblePush(offPtr, workRegister, TRUE);

	DWORD rnd = IntervalRand(0, 100);
	if(rnd < 50)
	{
		sprintf(instr, opsStr[MOV_REG], regs[workRegister], curAddress);
		Assemble(instr, LdrOffset, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}
	else
	{
		offPtr += AssemblePush(offPtr, curAddress, FALSE);
		offPtr += AssemblePop(offPtr, workRegister);
	}

	for(int i = numOps-1; i >= 0; i--)
	{
		if(ops[i].Operation == SWITCH_REG)
		{
			offPtr += AssemblePush(offPtr, workRegister, TRUE);

			sprintf(instr, "mov %s, [dword esp+4]", regs[workRegister]);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;

			do
			{
				workRegister = IntervalRand(0, 7);
			} while(workRegister == REG_ESP);

			sprintf(instr, "mov [dword esp+4], %s", regs[workRegister]);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;

			offPtr += AssemblePop(offPtr, workRegister);
		}
		else
		{
			sprintf(instr, opsStr[ops[i].Operation], regs[workRegister], ops[i].Operand);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;
		}
	}

	sprintf(instr, "add %s, [dword esp+8]", regs[workRegister]);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	offPtr += AssembleRet(offPtr, workRegister);

	free(ops);

	//the first stage loader has been built, we now assemble a "pop reg" as the first instruction of the next stage loader, this way the stack is restored
	//no polymorphism here, we need to know the exact size of the instruction
	DWORD nextLdrOffset = RvaToOffset(JmpAddress, pPEInfo->ImageNtHeaders);
	nextLdrOffset += (DWORD)pPEInfo->FileBase;

	sprintf(instr, "pop %s", regs[workRegister]);
	Assemble(instr, nextLdrOffset, &am, 0, 0, errtext);
	memcpy((PUCHAR)nextLdrOffset, am.code, am.length);

	return (DWORD)(offPtr - LdrOffset);
}

/*
the 1.5 loader prepares the stack for the real loader
the real loader gets encrypted with run-time generated operations, then the code necessary to
decrypt the real loader is generated
*/
DWORD __stdcall Build15Loader(PEInfo *pPEInfo, DWORD LdrOffset, DWORD RealLdrAddress, DWORD RealLdrSize, DWORD RealLdrDSize)
{
	OpInfo *ops;
	t_asmmodel am;
	ULONG numOps;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};
	PUCHAR realLdrOffset = (PUCHAR)RealLdrAddress;
	PUCHAR offPtr = (PUCHAR)LdrOffset;

	numOps = IntervalRand(1, 5); //maximum number of operations
	ops = (OpInfo *)malloc(sizeof(OpInfo)*numOps);
	for(ULONG i = 0; i < numOps; i++)
	{
		ops[i].Operation = IntervalRand(0, 4);
		if(ops[i].Operation == OP_ROL || ops[i].Operation == OP_ROR)
			ops[i].Operand = IntervalRand(1, 16);
		else
			ops[i].Operand = IntervalRand(0, 0xFE);
	}

	realLdrOffset = (PUCHAR)RvaToOffset(RealLdrAddress, pPEInfo->ImageNtHeaders);
	realLdrOffset = MakePtr(PUCHAR, realLdrOffset, pPEInfo->FileBase);

	for(ULONG i = 0; i < RealLdrSize; i++)
	{
		UCHAR curByte = *(realLdrOffset + i);
		for(ULONG j = 0; j < numOps; j++)
		{
			switch(ops[j].Operation)
			{
				case OP_ADD:
					curByte += (BYTE)ops[j].Operand;
					break;

				case OP_SUB:
					curByte -= (BYTE)ops[j].Operand;
					break;

				case OP_XOR:
					curByte ^= (BYTE)ops[j].Operand;
					break;

				case OP_ROL:
				{
					BYTE opr = (BYTE)ops[j].Operand;
					__asm {
						push eax
						push ecx
						movzx eax, curByte
						movzx ecx, opr
						rol al, cl
						mov curByte, al
						pop ecx
						pop eax
					}
				}
				break;

				case OP_ROR:
				{
					BYTE opr = (BYTE)ops[j].Operand;
					__asm {
						push eax
						push ecx
						movzx eax, curByte
						movzx ecx, opr
						ror al, cl
						mov curByte, al
						pop ecx
						pop eax
					}
				}
				break;
			}
		}

		*(realLdrOffset + i) = curByte;
	}

	DWORD rnd = IntervalRand(0, 100);
	if(rnd < 50)
		offPtr += AssemblePush(offPtr, IntervalRand(0, 7), TRUE);
	else
		offPtr += AssemblePush(offPtr, IntervalRand(0, 0xFFEEDDCC), FALSE);

	sprintf(instr, "pushfd");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "pushad");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov esi, 0%x", RealLdrAddress);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "add esi, [dword esp+2C]");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov ecx, 0%x", RealLdrSize);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	DWORD label1 = (DWORD)offPtr;
	for(int k = numOps - 1; k >= 0; k--)
	{
		sprintf(instr, opsStr[ops[k].Operation], "[byte esi]", ops[k].Operand);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	sprintf(instr, "inc esi");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "dec ecx");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "test ecx,ecx");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "jnz 0%x", label1);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov eax, [dword esp+2C]");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov esi, 0%x", RealLdrAddress);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "add esi, eax");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	offPtr += AssemblePush(offPtr, 6, TRUE);

	sprintf(instr, "add esi, 0%x", RealLdrDSize);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	offPtr += AssembleRet(offPtr, 6);

	return (DWORD)(offPtr - (PUCHAR)LdrOffset);
}

BOOL __stdcall EncryptCode(PEInfo *pPEInfo, PUCHAR Key)
{
	DWORD *codeStart;
	PIMAGE_SECTION_HEADER textSection;
	DWORD key[4];

	if(pPEInfo == NULL)
		return FALSE;

	textSection = IMAGE_FIRST_SECTION(pPEInfo->ImageNtHeaders);
	codeStart = MakePtr(DWORD *, pPEInfo->FileBase, textSection->PointerToRawData);

	//we can't change raw size of .text now, otherwise RvaToOffset and OffsetToRva will stop working
	g_dwNewRawSize = IntervalRand(0x11223344, 0xFFAABBCC); 

	key[0] = g_dwNewRawSize;
	key[1] = pPEInfo->ImageOptionalHeader->NumberOfRvaAndSizes;
	key[2] = *((DWORD *)(Key));
	key[3] = *((DWORD *)(Key + 4));

	for(ULONG i = 0; i < textSection->SizeOfRawData; i += 8)
	{
		if(codeStart[0] != 0 || codeStart[1] != 0)
			encipher(codeStart, codeStart, key);
		codeStart += 2;
	}

	return TRUE;
}

BOOL __stdcall EncryptData(PEInfo *pPEInfo, PUCHAR Key)
{
	DWORD *dataStart;
	PIMAGE_SECTION_HEADER dataSection;
	DWORD key[4];

	if(pPEInfo == NULL)
		return FALSE;

	dataSection = IMAGE_FIRST_SECTION(pPEInfo->ImageNtHeaders);
	while(strncmp((char *)dataSection->Name, ".data", 5) != 0)
		dataSection++;

	dataStart = MakePtr(DWORD *, pPEInfo->FileBase, dataSection->PointerToRawData);

	key[0] = g_dwNewRawSize;
	key[1] = *((DWORD *)(Key));
	key[2] = *((DWORD *)(Key+4));
	key[3] = *((DWORD *)(Key+8));

	for(ULONG i = 0; i < dataSection->SizeOfRawData; i += 8)
	{
		if(dataStart[0] != 0 || dataStart[1] != 0)
			encipher(dataStart, dataStart, key);
		dataStart += 2;
	}

	return TRUE;
}

BOOL __stdcall EncryptRData(PEInfo *pPEInfo, PUCHAR Key)
{
	DWORD *rdataStart;
	PIMAGE_SECTION_HEADER rdataSection;
	DWORD key[4];

	if(pPEInfo == NULL)
		return FALSE;

	rdataSection = IMAGE_FIRST_SECTION(pPEInfo->ImageNtHeaders);
	while(strncmp((char *)rdataSection->Name, ".rdata", 6) != 0)
		rdataSection++;

	rdataStart = MakePtr(DWORD *, pPEInfo->FileBase, rdataSection->PointerToRawData);

	key[0] = g_dwNewRawSize;
	key[1] = *((DWORD *)(Key));
	key[2] = *((DWORD *)(Key+4));
	key[3] = *((DWORD *)(Key+8));

	for(ULONG i = 0; i < rdataSection->SizeOfRawData; i += 8)
	{
		if(rdataStart[0] != 0 || rdataStart[1] != 0)
			encipher(rdataStart, rdataStart, key);
		rdataStart += 2;
	}

	return TRUE;
}

void encipher(DWORD *v, DWORD *w, DWORD *k)
{
	DWORD y = v[0], z = v[1], sum = 0, delta = 0x9E3779B9, n = 32;

	while(n-->0)
	{
		y += (z << 4 ^ z >> 5) + z ^ sum + k[sum&3];
		sum += delta;
		z += (y << 4 ^ y >> 5) + y ^ sum + k[sum>>11 & 3];
	}

	w[0]=y; w[1]=z;
}

void decipher(DWORD *v, DWORD *w, DWORD *k)
{
	DWORD y = v[0], z = v[1], sum = 0xC6EF3720, delta = 0x9E3779B9, n = 32;

	/* sum = delta<<5, in general sum = delta * n */

	while(n-->0)
	{
		z -= (y << 4 ^ y >> 5) + y ^ sum + k[sum>>11 & 3];
		sum -= delta;
		y -= (z << 4 ^ z >> 5) + z ^ sum + k[sum&3];
	}

	w[0]=y; w[1]=z;
}

//int _tmain(int argc, TCHAR *argv[])
void _polymer(TCHAR *argv1)
{
	PIMAGE_SECTION_HEADER pSection;
	srand((unsigned int)time(NULL));

	PEInfo *pPEInfo = LoadPE(argv1);
	g_LoaderInfo.pPEInfo = pPEInfo;
	RelocateFile(pPEInfo);
	EncryptReloc(pPEInfo);
	ProcessExportTable(pPEInfo);
	ContinueWork(&g_LoaderInfo);
	pSection = IMAGE_FIRST_SECTION(pPEInfo->ImageNtHeaders);
	pSection->SizeOfRawData = g_dwNewRawSize;
	UnloadPE(pPEInfo);
}

int main(int argc, char *argv[])
{
	_polymer((_TCHAR*)argv[1]);
	return 0;
}